@@ -3,7 +3,7 @@
 /*  Title       : satgraph.c                                                  */
 /*  Author      : Manfred Bester                                              */
 /*  Date        : 13Dec94                                                     */
-/*  Last change : 11Jan95                                                     */
+/*  Last change : 19Jan95                                                     */
 /*                                                                            */
 /*  Synopsis    : SatTrack graphics display routines.                         */
 /*                                                                            */
@@ -101,7 +101,7 @@ static XtInputMask    mask, lastMask;
 static Boolean        eventFlag;
 static Arg            warg[3];
 
-static double         gndLtd, gndLng, ctyLtd, ctyLng, satAzi, satEle, 
+static double         gndLtd, gndLng, ctyLtd, ctyLng, objEle, satAzi, satEle, 
                       satLtd, satLng, satHgt, mirLtd, mirLng, mirHgt, tdrsHgt, 
                       sepLtd, sepLng, sepHgt, sunLtd, sunLng, sunHgt, 
                       ltd, lng, hgt, visHgt, dH;
@@ -123,6 +123,8 @@ static int            x, y, theScreen, nPoints, initFlag, mapSize, gndSpot,
                       gndLocX, gndLocY, ctyLocX, ctyLocY, satLocX, satLocY, 
                       mirLocX, mirLocY, sunLocX, sunLocY, argC;
 
+static int            failFlag = FALSE;
+
 char                  idStr[80], str[80], str1[120], str2[120], longStr[120], 
                       appName[80], selName[40], tdrsName[40], *argV[5];
 
@@ -162,13 +164,13 @@ char *nStr;
 
     for (i = 0; i < nPoints; i++)
     {
-        if (!orbFlag)
+        if (!orbFlag)                                  /* on-orbit attitude   */
         {
             stsPoints[i].x =  stsPointsWhite[i].x + (short int) satLocX;
             stsPoints[i].y =  stsPointsWhite[i].y + (short int) satLocY;
         }
 
-        else /* pre-launch configuration */
+        else                                           /* pre-launch attitude */
         {
             stsPoints[i].x =  stsPointsWhite[i].y + (short int) satLocX;
             stsPoints[i].y = -stsPointsWhite[i].x + (short int) satLocY;
@@ -189,23 +191,24 @@ char *nStr;
 
     for (i = 0; i < nPoints; i++)
     {
-        if (!orbFlag)
+        if (!orbFlag)                                  /* on-orbit attitude   */
         {
             stsPoints[i].x =  stsPointsBlack[i].x + (short int) satLocX;
             stsPoints[i].y =  stsPointsBlack[i].y + (short int) satLocY;
         }
 
-        else                                      /* pre-launch configuration */
+        else                                           /* pre-launch attitude */
         {
             stsPoints[i].x =  stsPointsBlack[i].y + (short int) satLocX;
             stsPoints[i].y = -stsPointsBlack[i].x + (short int) satLocY;
         }
-
     }
 
     XSetForeground(theDisplay,theGC,blackPixel);
     XDrawPoints(theDisplay,drawable,theGC,stsPoints,nPoints,CoordModeOrigin);
 
+    /* show mission number if several shuttles are in the multisat list */
+
     if (nShuttle > 1)
     {
         if (!orbFlag)
@@ -275,7 +278,7 @@ int dayFl;
 
 /******************************************************************************/
 /*                                                                            */
-/* DrawSat: draws satellite                                                   */
+/* DrawSat: draws satellite other than Mir or STS                             */
 /*                                                                            */
 /*          selFl = TRUE    color is blue   (status = SELECT or MARK)         */
 /*                = FALSE   color is yellow                                   */
@@ -327,8 +330,8 @@ char *nStr;
 
 /******************************************************************************/
 /*                                                                            */
-/* DrawFrame: draws the frame around world map and the text in the header and */
-/*            footer line                                                     */
+/* DrawFrame: draws the black frame around world map and the text in the      */
+/*            header and footer line                                          */
 /*                                                                            */
 /******************************************************************************/
 
@@ -343,14 +346,49 @@ void DrawFrame()
     XSetForeground(theDisplay,theGC,blackPixel);
     XFillRectangle(theDisplay,drawable,theGC,x,y,w,h);
 
-    XSetForeground(theDisplay,theGC,whitePixel);
+    if (trackObject == SAT)
+    {
+        sprintf(str,"%s",selName);
+        objEle = satEle;
+    }
 
-    sprintf(str,"%s",selName);
-    XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+2),
+    if (trackObject == SUN)
+    {
+        sprintf(str,"Sun");
+        objEle = sunElevation * CRD;
+    }
+
+    if (trackObject == MOON)
+    {
+        sprintf(str,"Moon");
+        objEle = moonElevation * CRD;
+    }
+
+    if (trackingFlag)
+    {
+        if (objEle > ONEPPM)
+            XSetForeground(theDisplay,theGC,lightGreenPixel);
+        else
+            XSetForeground(theDisplay,theGC,redPixel);
+
+        x = 12;
+
+        XFillRectangle(theDisplay,drawable,theGC,
+                   (int) (MARGINLFT+1),(int) (MARGINTOP-14),
+                   (unsigned int) (9),(unsigned int) (9));
+    }
+
+    XSetForeground(theDisplay,theGC,whitePixel);
+    XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+2+x),
                 (int) HEADERROW,str,(int) strlen(str));
 
     sprintf(str,"Orbit: %ld",selOrbitNum);
-    XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.17*MAPWIDTH),
+
+    if (mapSize == SMALL)
+        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.24*MAPWIDTH),
+                (int) HEADERROW,str,(int) strlen(str));
+    else
+        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.18*MAPWIDTH),
                 (int) HEADERROW,str,(int) strlen(str));
 
     if (mapSize == LARGE)
@@ -360,7 +398,7 @@ void DrawFrame()
         sprintf(str,"Azi: %3.0f  Ele: %3.0f",satAzi,satEle);
 
     if (mapSize == LARGE || mapSize == MEDIUM)
-        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.32*MAPWIDTH),
+        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.33*MAPWIDTH),
                (int) HEADERROW,str,(int) strlen(str));
 
     if (mapSize == LARGE)
@@ -381,7 +419,7 @@ void DrawFrame()
         sprintf(str2,"%s  Lng: %3.0f %s",str1,fabs(satLng),
                (satLng >= 0.0) ? "W":"E");
 
-        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.53*MAPWIDTH),
+        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.54*MAPWIDTH),
                (int) HEADERROW,str2,(int) strlen(str2));
     }
 
@@ -392,7 +430,7 @@ void DrawFrame()
         sprintf(str2,"%s  Lng: %3.0f %s",str1,fabs(satLng),
                (satLng >= 0.0) ? "W":"E");
 
-        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.40*MAPWIDTH),
+        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.43*MAPWIDTH),
                (int) HEADERROW,str2,(int) strlen(str2));
     }
 
@@ -406,7 +444,7 @@ void DrawFrame()
     if (mapSize == SMALL)
     {
         sprintf(str,"%s",xTimeStr);
-        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.72*MAPWIDTH),
+        XDrawString(theDisplay,drawable,theGC,(int) (MARGINLFT+0.73*MAPWIDTH),
                (int) HEADERROW,str,(int) strlen(str));
     }
 
@@ -465,7 +503,7 @@ void DrawFrame()
 
 /******************************************************************************/
 /*                                                                            */
-/* DrawMap: draws window with world map and other features                    */
+/* DrawMap: draws graphics window with world map and other features           */
 /*                                                                            */
 /******************************************************************************/
 
@@ -475,6 +513,8 @@ void DrawMap()
     int i, nSats;
 
     /* load initial satellite data and check if Mir is in the list */
+    /* (the latter is needed for STS-Mir docking maneuvers)        */
+    /* also count number of space shuttles in the list             */
 
     mirFlag  = FALSE;
     metFlag  = FALSE;
@@ -537,8 +577,9 @@ void DrawMap()
 
     else
     {
-        craFlag = satCrashFlag;
+        lchFlag = launchFlag;
         orbFlag = preOrbitFlag;
+        craFlag = satCrashFlag;
 
         satLtd  = satLat * CRD;
         satLng  = satLong * CRD;
@@ -553,7 +594,7 @@ void DrawMap()
         if (satTypeFlag == STS)
         {
             selOrbitNum = stsOrbitNum;
-            metFlag = (strlen(xMetStr) && launchFlag) ? TRUE : FALSE;
+            metFlag = (strlen(xMetStr) && lchFlag) ? TRUE : FALSE;
         }
 
         else
@@ -582,8 +623,8 @@ void DrawMap()
 
     if (debugFlag)
     {
-        sunLtd =  0.0;
-        sunLng =  0.0;
+        sunLtd = ZERO;
+        sunLng = ZERO;
 
         groundStation[0].ltd =  0.0 * CDR;
         groundStation[0].lng = 60.0 * CDR;
@@ -608,7 +649,7 @@ void DrawMap()
     XDrawSegments(theDisplay,drawable,theGC,gridLinesX,NGRIDX*2);
     XDrawSegments(theDisplay,drawable,theGC,gridLinesY,NGRIDY*2);
 
-    /* draw frame with header and footer lines */
+    /* draw black frame with header and footer lines */
 
     DrawFrame();
 
@@ -752,6 +793,8 @@ void DrawMap()
             sprintf(str,"%s",satName);
         }
 
+        /* check if satellite to be drawn is TDRS */
+
         tdrsFlag = (!strncmp(tdrsName,"TDRS",4)) ? TRUE : FALSE;
 
         if ((selFlag && !orbFlag) || (!selFlag && tdrsFlag))
@@ -816,10 +859,14 @@ void DrawMap()
                     sepLtd = fabs(ltd - mirLtd);
                     sepHgt = fabs(hgt - mirHgt);
 
+                    /* if separation of STS and Mir is less than 5 deg and */
+                    /* the height difference is less than 5 km:            */
+                    /* show Mir and STS docked together                    */
+
                     if ((sepLng < 5.0 || sepLng > 355.0) && sepLtd < 5.0 && 
                          sepHgt < 10.0)
                     {
-                        satLocX = mirLocX -  8;            /* Mir-STS docking */
+                        satLocX = mirLocX -  8;
                         satLocY = mirLocY + 16;
                     }
                 }
@@ -1319,10 +1366,14 @@ int      numSegments, visibCircleFlag;
             dX = (int) xSegments[k].x2 - (int) MARGINLFT + dx;
 
             dY = (int) (xSegments[k].y2 - xSegments[k].y1);
-            dy = (int) ((double) dY / (double) dX * (double) dx);
+
+            if (dX != 0)
+                dy = (int) ((double) dY / (double) dX * (double) dx);
+            else
+                dy = 0;
 
             xSegments[n].x1 = xSegments[k].x2 - (short int) (dX - dx);
-            xSegments[k].x2 = xSegments[k].x1 + (short int) dx; /* - 0.5 */
+            xSegments[k].x2 = xSegments[k].x1 + (short int) dx;
 
             xSegments[n].y1 = xSegments[k].y2 - (short int) (dY - dy);
             xSegments[k].y2 = xSegments[k].y1 + (short int) dy;
@@ -1334,10 +1385,14 @@ int      numSegments, visibCircleFlag;
             dX = (int) (MAPWIDTH + MARGINLFT) - (int) xSegments[k].x2 + dx;
 
             dY = (int) (xSegments[k].y2 - xSegments[k].y1);
-            dy = (int) ((double) dY / (double) dX * (double) dx);
+
+            if (dX != 0)
+                dy = (int) ((double) dY / (double) dX * (double) dx);
+            else
+                dy = 0;
 
             xSegments[n].x1 = xSegments[k].x2 + (short int) (dX - dx);
-            xSegments[k].x2 = xSegments[k].x1 - (short int) dx; /* - 0.5 */
+            xSegments[k].x2 = xSegments[k].x1 - (short int) dx;
 
             xSegments[n].y1 = xSegments[k].y2 - (short int) (dY - dy);
             xSegments[k].y2 = xSegments[k].y1 + (short int) dy;
@@ -1388,12 +1443,18 @@ int StartGraphics()
 {
     int  n;
 
+    if (failFlag)
+    {
+        dispMessage("Attempt to open graphics window failed before.");
+        return(FALSE);
+    }
+
     initFlag = TRUE;
 
     if (firstGraphics)
     {
-        lastSatGraphHeight = 0.0;
-        lastSunGraphTime   = 0.0;
+        lastSatGraphHeight = -2.0 * DHLIMIT;
+        lastSunGraphTime   = ZERO;
         numGroundStations  = 1;
 
         argV[0] = "sattrack";
@@ -1410,7 +1471,8 @@ int StartGraphics()
 
     if (!theDisplay)
     {
-        dispMessage("Tried to open graphics window - failed.");
+        dispMessage("Attempt to open graphics window failed.");
+        failFlag = TRUE;
         return(FALSE);
     }
 
@@ -1447,14 +1509,11 @@ int StartGraphics()
     frame = XtCreateManagedWidget("frame",widgetClass,topLevel,warg,n);
 
     XtAppAddTimeOut(appContext,1L,TimeEvent,0);
+
     XtAddEventHandler(frame,StructureNotifyMask,FALSE,ResizeMap,NULL);
     XtAddEventHandler(frame,VisibilityChangeMask,FALSE,RedrawMap,NULL);
     XtAddEventHandler(frame,ExposureMask,FALSE,RedrawMap,NULL);
 
-/*
-    XtAddEventHandler(frame,SubstructureNotifyMask,FALSE,ResizeMap,NULL);
-*/
-
     XtRealizeWidget(topLevel);
 
     theWindow = XtWindow(topLevel);
@@ -1572,9 +1631,11 @@ int strtFlag;
         /* it has not been dispatched yet, dispatch it now            */
 
         if (mask != lastMask && mask != (unsigned long) 2)
+        {
             eventFlag = XtDispatchEvent(&theEvent);
+            lastMask  = mask;
+        }
 
-        lastMask = mask;
         nEvents++;
     }
 
