@@ -3,7 +3,7 @@
 /*  Title       : sattime.c                                                   */
 /*  Author      : Manfred Bester                                              */
 /*  Date        : 04Dec91                                                     */
-/*  Last change : 05Jan95                                                     */
+/*  Last change : 14Jan95                                                     */
 /*                                                                            */
 /*  Synopsis    : This function block contains auxiliary routines dealing     */
 /*                with time and date.                                         */
@@ -116,12 +116,11 @@ double timeArg;
 
 {
     double tu, tusq, tucb, utcInterval, julianDay, rss;
-    long   jd;
-    
+
+    utcInterval = modf(timeArg,&dummyd);                               /* [d] */
+
     julianDate  = timeArg + JULDAT1900 - 0.5;                          /* [d] */
-    julianDay   = (double) ((long) julianDate) + 0.5;                  /* [d] */
-    utcInterval = julianDate - julianDay;                              /* [d] */
-    utcInterval = reduce(utcInterval,ZERO,ONE);                        /* [d] */
+    julianDay   = julianDate - utcInterval;                            /* [d] */
 
     tu          = (julianDay - JULDAT2000) / JULCENT;                /* [jcy] */
     tusq        = tu*tu;
